/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/

package robotcontroller;

/**
 * Example
 * <pre>
 * Alarm contestIsOver = new Alarm(60000, "games up") {
 *   // override the timesup method
 *   public void timesup() {
 *      shut down motors
 *      set system state to quiesce
 *   };
 * };
 * </pre>
 */
public class Alarm implements Runnable, AlarmListener {
    long whenMillis = 0;
    Thread thread;
    String alarmName;

    /**
     * If true, then each Alarm creates an associated thread that sleeps
     * until the deadline and then calls <code>timesup</code>.  If false,
     * then each Alarm adds itself to Controller as an
     * <code>AlarmListener</code> so that it will be called from the
     * <code>run</code> main loop.
     */
    public static boolean useThreadsForAlarms = false;

    /**
     * Creates a new alarm object that will call it's <code>timesup</code> method in whenMillis milliseconds.
     * @param whenMillis How many milliseconds from now to call <code>timesup</code> 
     * @param alarmName The name of this alarm
     */
    public Alarm(long whenMillis, String alarmName) {
        long ctm = System.currentTimeMillis();
        this.whenMillis = whenMillis + ctm;
        this.alarmName = alarmName;

        if (useThreadsForAlarms) {
            thread = new Thread(this, alarmName);
            thread.setDaemon(true); /* don't impede JVM termination */
            thread.start();
        } else {
            Controller.getController().addAlarmListener(this);
        }
    };

    /**
     * Returns the millisecond time deadline of this alarm.
     */
    public long getDeadline() {
        return whenMillis;
    }

    /**
     * This method is called when the alarm goes off.
     *
     * The user's alarm object should override this method.
     */
    public void timesup() throws ControllerException {
        /* do nothing by default */
    }
    /**
     * This method does the work.  It is invoked by the internals of Thread
     * if <code>useThreadsForAlarms</code> is true.  The user should not
     * invoke this method.
     */
    public void run() {
        while (true) {
            long ctm = System.currentTimeMillis();
            if (whenMillis <= ctm) {
                try {
                    timesup();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return; /* done */
            } else {
                try {
                    Thread.sleep(whenMillis - ctm);
                } catch (InterruptedException e) {
                    /* ok, spin again */
                }
            }
        }
    }
};
