/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/


package robotcontroller;

/**
 * Represents one input to the analog to digital converter.  There are 32
 * inputs to the converter.
 *
 * The first 24 ports are numbered as one would expect, from 0 to 23.
 * However, <em>ports 24 through 31 have a non-intuitive mapping.</em>
 * Symbolic constants are provided for all AnalogInput ports:
 * <code>AIN0_PORT</code> through <code>AIN31_PORT</code> with the correct
 * values.  See the documentation associated with these values for a
 * description of what is connected.
 * 
 * The daughtercard is continuously performing conversions.  The static
 * method <code>updateValues</code> fetches the most recent values from the
 * daughtercard.  Once they are fetched, the value for each port may be
 * accessed via <code>getValue</code>.  
 *
 * If <code>Controller.run</code> (recommended) or <code>Controller.startUserThread</code> (not recommended) is invoked then
 * user code does not have to invoke <code>updateValues</code>, it is
 * automatically invoked once per <code>Controller.cycleTime</code>
 * milliseconds by the Controller thread.
 *
 *
 * The Controller daughtercard's firmware is designed to rapidly sample all its
 * analog ports as quickly as possible. This process currently takes approx.
 * 5ms for all 32 ports.  The first 8 ports (sensors 0-7) are further augemented
 * with the ability to act as shaft encoder ports that count transitions from
 * low to high.
 *
 **/
public class AnalogPort implements UpdateListener {
    static byte values[] = new byte[Controller.ADC_NUMPORTS];
    static int portsInUse = 0;

    static Controller rdc;
    int port;

    /**
     * Input pin 0 on connector J5
     */
    public static final int AIN0_PORT = 0;
    /**
     * Input pin 1 on connector J5
     */
    public static final int AIN1_PORT = 1;
    /**
     * Input pin 2 on connector J5
     */
    public static final int AIN2_PORT = 2;
    /**
     * Input pin 3 on connector J5
     */
    public static final int AIN3_PORT = 3;
    /**
     * Input pin 4 on connector J5
     */
    public static final int AIN4_PORT = 4;
    /**
     * Input pin 5 on connector J5
     */
    public static final int AIN5_PORT = 5;
    /**
     * Input pin 6 on connector J5
     */
    public static final int AIN6_PORT = 6;
    /**
     * Input pin 7 on connector J5
     */
    public static final int AIN7_PORT = 7;
    /**
     * Input pin 8 on connector J5
     */
    public static final int AIN8_PORT = 8;
    /**
     * Input pin 9 on connector J5
     */
    public static final int AIN9_PORT = 9;
    /**
     * Input pin 10 on connector J5
     */
    public static final int AIN10_PORT = 10;
    /**
     * Input pin 11 on connector J5
     */
    public static final int AIN11_PORT = 11;
    /**
     * Input pin 12 on connector J5
     */
    public static final int AIN12_PORT = 12;
    /**
     * Input pin 13 on connector J5
     */
    public static final int AIN13_PORT = 13;
    /**
     * Input pin 14 on connector J5
     */
    public static final int AIN14_PORT = 14;
    /**
     * Input pin 15 on connector J5
     */
    public static final int AIN15_PORT = 15;
    /**
     * Input pin 16 on connector J5
     */
    public static final int AIN16_PORT = 16;
    /**
     * Input pin 17 on connector J5
     */
    public static final int AIN17_PORT = 17;
    /**
     * Input pin 18 on connector J5
     */
    public static final int AIN18_PORT = 18;
    /**
     * Input pin 19 on connector J5
     */
    public static final int AIN19_PORT = 19;
    /**
     * Input pin 20 on connector J5
     */
    public static final int AIN20_PORT = 20;
    /**
     * Input pin 21 on connector J5
     */
    public static final int AIN21_PORT = 21;
    /**
     * Input pin 22 on connector J5
     */
    public static final int AIN22_PORT = 22;
    /**
     * Input pin 23 on connector J5
     */
    public static final int AIN23_PORT = 23;
    /**
     * Input pin 24 on connector J5, analog input number <em>31</em>
     */
    public static final int AIN24_PORT = 31;
    /**
     * Potentiometer RV1, labeled AIN25, analog input number <em>30</em>
     */
    public static final int AIN25_PORT = 30;
    /**
     * Potentiometer RV2, labeled AIN26, analog input number <em>29</em>
     */
    public static final int AIN26_PORT = 29;
    /**
     * Pushbutton S2, labeled AIN27, analog input number <em>28</em>
     */
    public static final int AIN27_PORT = 28;
    /**
     * Pushbutton S3, labeled AIN28, analog input number <em>27</em>
     */
    public static final int AIN28_PORT = 27;
    /**
     * Pushbutton S4, labeled AIN29, analog input number <em>26</em>
     */
    public static final int AIN29_PORT = 26;
    /**
     * Pushbutton S1, labeled AIN30, analog input number <em>25</em>
     */
    public static final int AIN30_PORT = 25;
    /**
     * The 6V supply is wired to AIN31, analog input number <em>24</em>
     */
    public static final int AIN31_PORT = 24;

    /**
     * Creates a handle for the specified port of the analog to digital converter.
     * @exception ControllerException if the port was already allocated (by another instance of <code>AnalogPort</code>).
     */
    public AnalogPort(int port) throws ControllerException
    {
        this.port = port;
        if (rdc == null) {
            rdc = Controller.getController();
            rdc.addUpdateListener(this);
        }
        int portMask = (1 << port);
        if ((portMask & portsInUse) != 0)
            throw new ControllerException("adc port already in use: " + Long.toString(portMask) + " " + Long.toString(portsInUse));
        portsInUse |= portMask;
    };

    /**
     * frees this port
     */
    public synchronized void finalize()
    {
        portsInUse &= ~(1 << port);
    };

    /**
     * gets the value associated with this port from the last call to <code>updateValues</code>
     */
    public synchronized byte getValue()
    {
        if (values == null) {
            System.err.println("AnalogPort.getValue: values array is null");
        }
        return values[port];
    }

    /**
     * fetches the most recent conversion values from the daughtercard.
     * 
     * This updates the values for all AnalogPorts.  Note that Controller only cycles through all the analog ports once every 5ms.
     *
     *
     * User code does not have to invoke <code>updateValues</code>, it is
     * automatically invoked once per <code>Controller.valueUpdateInterval</code>
     * milliseconds by the Controller thread or by Controller.run.
     * @exception ControllerError if there is a problem accessing the daughtercard.
     */
    public synchronized static void updateValues() throws ControllerError
    {
        rdc.adc_get_values(values);
    }

    public synchronized void update() throws ControllerError
    {
        // System.err.println("  AnalogPort.update calling adc_get_values");
        rdc.adc_get_values(values);
    }
};
