/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/

package robotcontroller;

/**
 * A behavior encapsulates the bits of reflexive code that should go in the
 * robot's main loop, allowing them to be enabled, disabled, and
 * overriden without requiring too much boilerplate code.
 */
public class Behavior implements BehaveListener {
    static Behavior behaviors[] = new Behavior[128];
    static int nBehaviors = 0;


    String behaviorName = null;
    boolean enabled = false;
    Behavior overriding = null;
    Behavior overriddenBy = null;


    /**
     * disables all behaviors.
     */
    public static void disableAll() {
	for (int b = 0; b < nBehaviors; b++) {
	    behaviors[b].disable();
	}
    };

    /**
     * Creates a new behavior object and registers it with Controller so that
     * it will be called once per cycle of run as long as it is enabled.
     * Behaviors are invoked after sensors are updated.
     *
     * @param behaviorName The name of this behavior */
    public Behavior(String behaviorName) {
        this.behaviorName = behaviorName;
        behaviors[nBehaviors++] = this;
        Controller.getController().addBehaveListener(this);
    };

    /*
     * returns the name of this behavior.
     */
    public String getName() { return behaviorName; };

    /**
     * Called once each cycle while the behavior is enabled, after all the
     * sensor update methods are called.  Does nothing by default.
     *
     * The user's behavior object should override this method.  */
    public void behave() throws ControllerException {
        /* do nothing by default */
    };

    /**
     * returns <code>true</code> if the behavior is enabled.
     */
    public boolean isEnabled()
    {
        return enabled;
    };

    /**
     * Enables this behavior
     */
    public void enable()
    {
        enabled = true;
    };

    /**
     * Disables this behavior
     */
    public void disable()
    {
        enabled = false;
	if (overriding != null) {
	    overriding.enable();
	    overriding = null;
	}
    };

    public synchronized void override(Behavior activeBehavior) throws ControllerException
    {
        if (!activeBehavior.isEnabled())
            throw new ControllerException("Behavior " + activeBehavior.getName() + " is inactive -- can't override");
        if (enabled)
            throw new ControllerException("Behavior " + getName() + " is active -- can't override another behavior");
        if (overriding != null)
            throw new ControllerException("Behavior " + getName() 
                                         + " is already overriding " + activeBehavior.getName()
                                         + " -- can't override another behavior");
        overriding = activeBehavior;
        activeBehavior.enabled = false;
        enabled = true;
    };

};
