/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/

/*** Controller Access Routines vis USB
 ***
 *** All values passed to these routines are either ints or byte arrays.
 ***     [ We could switch to chars instead of int's if you'd prefer.]
 *** Every routine throws ControllerError if a failure occurs.
 ***
 ***/
package robotcontroller;

/**
 * <code>Controller</code> provides raw access to skfrdc via USB.  Most applications can use the other classes in this package.
 *
 * Controller is a singleton class -- only one instance of it may be created,
 * corresponding to the physical Controller.  For that reason, the constructor
 * is private.  Construct a Controller by calling <code>getController</code>
 *
 * Over time, the native methods may be migrated to the corresponding wrapper classes.  */
public class Controller implements Runnable {
    /******* CHANGE THESE DEFS TO BE AN INCLUDE OF ROBODC.H  !!!!!!!!1 *******/
   
    /**
     * The number of ports on the analog-to-digital converter
     */
    public static final int ADC_NUMPORTS = 32;
    /**
     * The number of shaft encoder ports
     */
    public static final int ENC_NUMPORTS = 8;
    /**
     * The number of servo ports.
     */
    public static final int SERVO_NUMPORTS = 4;
    /**
     * The length of the pulse-width modulation vector for each motor
     */
    public static final int MTR_VECLEN = 8;
    /**************************************************************/
   
    private static Controller rdc;
    private Thread controllerThread;

    /**
     * Creates a new singleton Controller.
     */
    private Controller() throws ControllerError {
        usb_init();
        rdc = this;
    };
   
    /**
     * starts the Controller thread.  When this thread is running, users do not
     * have to invoke AnalogPort.updateValues or Encoder.updateAllCounts.  
     * 
     * spawns a thread that handles the alarms, update and behave listeners once every <code>cycleTime</code> milliseconds.
     *
     * <em>It is recommended that users do not start this thread, but invoked
     * Controller.run directly, so that synchronization will not be an issue.</em>
     */
    public void startControllerThread() {
        try {
            controllerThread = new Thread(this, "Controller update thread");
            controllerThread.setDaemon(true); /* allow JVM to exit even if the update thread is still running */
            controllerThread.start();
            System.err.println("Controller update thread started");
        } catch (Exception e) {
            System.err.println("robotcontroller.Controller caught an exception while creating the controllerThread");
            e.printStackTrace();
            throw new ControllerError("failed to create controllerThread");
        }
    };

    /**
     * Returns the singleton instance of Controller.  Initializes the USB connection to skfrdc if it has not already been done.
     *
     * 
     */
    public synchronized static Controller getController() throws ControllerError
    {
        if (rdc != null) {
            return rdc;
        } else {
            return new Controller();
        }
    }

    /**
     * Not implemented yet.
     */
    public synchronized void downloadFirmware(String firmwareFilename) throws ControllerException {
        byte data[];

        throw new ControllerException("download firmware unimplemented");

        // put 8051 in reset
        // aram_write(1, 0x7f92, "01");
      
        // parse hex records and write to aram
        // aram_write(len, addr, data);
        // release 8051 from reset
        // aram_write(1, 0x7f92, "00");
    }

    /**
     * Opens the usb port
     *
     * @exception ControllerError if a failure is detected.
     */ 
    private native void usb_init() throws ControllerError;
   
   
    /**
     * Reads from a region in first 4KB of the EZUSB RAM.
     *
     *   @param addr aram address to be read
     *   @param len  num bytes to be read
     *   @param data buffer to which result data is written
     *
     * @exception ControllerError if a failure is detected.
     */ 
    public synchronized native void aram_read(int addr, int len, byte data[]) throws ControllerError;
    /**
     * Writes to region in first 4KB of the EZUSB RAM.
     *
     *   @param addr  aram to be written
     *   @param len   the number of bytes to be written
     *   @param data  buffer from which data is read
     *
     * @exception ControllerError if a failure is detected.
     */ 
    public synchronized native void aram_write(int addr, int len, byte data[]) throws ControllerError;
   
    /**
     * Reads from the eeprom.
     *
     *   @param addr  eeprom address to be read
     *   @param len   the number of bytes to be read
     *   @param buffer into which result data will be written
     *
     * @exception ControllerError if a failure is detected.
     */ 
    public synchronized native void eeprom_read(int addr, int len, byte data[]) throws ControllerError;
    /**
     * Writes to the eeprom.
     *
     *   @param addr  eeprom address to be written
     *   @param len   the number of bytes to be read
     *   @param buffer from which data is read
     *
     * @exception ControllerError if a failure is detected.
     */ 
    public synchronized native void eeprom_write(int addr, int len, byte data[]) throws ControllerError;
   
    /**
     * Get I2C width.
     *
     * Returns byte width of eeprom (0 if 1 byte, 1 if 2 bytes)
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native int get_eeprom_width() throws ControllerError;
   
    /**
     * Get USB Chip revision.
     *
     * Returns the EZUSB chip revision. (Rev A,B = 0, Rev C = 2, Rev D = 3)
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native int get_usb_chip_rev() throws ControllerError;
   
   
    /**
     * Disconnects and renumerates one second from receipt of command.
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void usb_renumerate();
   
   
    /**
     * Reads the values of the A2D converters. 
     *
     * (The HW updates these values 
     * approx 100 times per second.)
     * @param data pointer to a result buffer of length ADC_NUMPORTS.
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native int adc_get_values(byte data[]) throws ControllerError;
   
    /**
     * Set an entire motor register vector.
     *
     * @param reg   the motor register to be set, should be 0 or 1.
     * @param data   buffer of length MTR_VECLEN.
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void set_motor_vec(int reg, byte data[]) throws ControllerError;
   
   
    /**
     * Enable Servo Port
     *
     *   Enables timer interrupts for control of a particular servo
     *   port. Note, these ports take a huge amount of processor time to
     *   control, so only enable the ones you are actually using.
     *
     *   @param port  port number  [port between 0 and <code>SERVO_NUMPORTS</code>]
     *   @param state   0 (off) or 1 (on) 
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void servo_enable_port (int port, int state) throws ControllerError;
   
    /**
     * Set Servo Port
     *   Configures the length of the pulse generated on a servo port (and
     *   therefore sets the position the servo tries to keep itself at).
     *
     *   Note, there is no error checking; maximum flexibility is given. You
     *   <EM>CAN</EM> damage your servo or even destroy it by feeding it range
     *   values which are too high or too low.
     *
     *  @param port   port number
     *  @param period timer period (max (2^16)-1)
     *        range of useful values: (constants are defined for these)
     *          60928 - left
     *          63036 - middle
     *          65056 - right
     *
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void servo_set_port (int port, int period) throws ControllerError;
   
    /**
     * timer period that will cause a servo to move all the way to the left.
     */
    public static final int SERVO_MAX_LEFT = 60928;
    /**
     * timer period that will cause a servo to move to the middle.
     */
    public static final int SERVO_MIDDLE = 63036;
    /**
     * timer period that will cause a servo to move all the way to the right.
     */
    public static final int SERVO_MAX_RIGHT = 65056;
   
   
    /**
     * Enables/disables all encoders
     *
     *  @param state   0 (to disable) or 1 (to enable)
     *        Note that you cannot disable/enable individual encoder counters
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void encoder_enable_all(int state) throws ControllerError;
   
    /**
     * Configures an encoder
     *
     *  @param port   Number of encoder to be configured
     *  @param low    ??
     *  @param high   ??
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void encoder_configure(int port, int low, int high) throws ControllerError;

    /**
     * Reads all encoder counts (one per encoder port)
     *
     *   @param data  result buffer of length ENC_NUMPORTS.
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void encoder_get_counts(byte data[]) throws ControllerError;

    /**
     * Resets all the specified encoder's counts
     *
     * @param port specifies which encoder to reset
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void encoder_reset_count(int port) throws ControllerError;

    /**
     * Resets all encoder counts
     *
     * @exception ControllerError if a failure is detected.
     */
    public synchronized native void encoder_reset_counts() throws ControllerError;

    /**
     * Controls how often <code>AnalogPort.updateValues</code> and
     * <code>Encoder.UpdateAllCounts</code> are invoked.  In milliseconds.  
     */
    public int cycleTime = 1000;

    private int alarmListenerCount = 0;
    private AlarmListener alarmListeners[] = new AlarmListener[32];

    public synchronized void addAlarmListener(AlarmListener listener) {
        if (alarmListenerCount >= alarmListeners.length) {
            AlarmListener newl[] = new AlarmListener[2 * alarmListeners.length];
            for (int i = 0; i < alarmListeners.length; i++)
                newl[i] = alarmListeners[i];
            alarmListeners = newl;
        }
        alarmListeners[alarmListenerCount++] = listener;
    }

    private int updateListenerCount = 0;
    private UpdateListener updateListeners[] = new UpdateListener[32];

    public synchronized void addUpdateListener(UpdateListener listener) {
        if (updateListenerCount >= updateListeners.length) {
            UpdateListener newul[] = new UpdateListener[2 * updateListeners.length];
            for (int i = 0; i < updateListeners.length; i++)
                newul[i] = updateListeners[i];
            updateListeners = newul;
        }
        updateListeners[updateListenerCount++] = listener;
    }

    private int behaveListenerCount = 0;
    private BehaveListener behaveListeners[] = new BehaveListener[32];

    public synchronized void addBehaveListener(BehaveListener listener) {
        if (behaveListenerCount >= behaveListeners.length) {
            BehaveListener newl[] = new BehaveListener[2 * behaveListeners.length];
            for (int i = 0; i < behaveListeners.length; i++)
                newl[i] = behaveListeners[i];
            behaveListeners = newl;
        }
        behaveListeners[behaveListenerCount++] = listener;
    }

    /**
     * If true, then prints the names of updateListeners and behaveListeners as they are invoked.
     */
    boolean verbose = false;
    /**
     * Sets verbosity of the <code>run</code> method.  If true, then prints
     * the names of updateListeners and behaveListeners as they are
     * invoked.
     */
    public void setVerbose(boolean newVerbose) { 
        verbose = newVerbose; 
    };

    public boolean exitOnListenerException = false;
    /**
     * indicates whether to exit on catching an exception from an update or behave listener.  (Default is not to exit.)
     */
    public void setExitOnListenerException(boolean newEOLE) {
        exitOnListenerException = newEOLE;
    }

    /**
     * <Returns current value of <code>exitOnListenerException</code>.
     */
    public boolean getExitOnListenerException() {
        return exitOnListenerException;
    }
    
    /**
     * Is the main loop of the program: each cycle, it runs the
     * <code>timesup</code> method of any alarms whose deadline has been
     * reached, then it invokes the <code>update</code> method of all the
     * registered updateListeners, followed by invoking the
     * <code>behave</code> method of all the registered and enabled
     * behaveListeners.
     * 
     * Catches and prints <code>Exceptions</code> caught while invoking the listeners.  If 
     * 
     * If <code>setVerbose(true)</code>, then it prints the names of the listeners before invoking them.
     * 
     * If <code>startControllerThread</code> is called, then this method is invoked from the
     * Thread associated with Controller.  Otherwise, the user should invoke it
     * as the main part of the robot code.
     */
    public void run() {
        if (verbose) System.err.println("Controller.run entered");
        while (true) {
            long ctm = System.currentTimeMillis();
            
            int acount = alarmListenerCount;
            for (int i = 0; i < acount; i++) {
                AlarmListener al = alarmListeners[i];
                if (al != null && al.getDeadline() < ctm) {
                    try {
                        al.timesup();
                    } catch (Exception t) {
                        t.printStackTrace();
                        if (exitOnListenerException)
                            return;
                    }
                    alarmListeners[i] = null;
                }
            }

            int count = updateListenerCount;
            for (int i = 0; i < count; i++) {
                try {
                    if (verbose) System.err.println("Controller.run: invoking updateListener " + updateListeners[i].toString());
                    updateListeners[i].update();
                } catch (Exception t) {
                    t.printStackTrace();
                    if (exitOnListenerException)
                        return;
                }
            }


            int bcount = behaveListenerCount;
            for (int i = 0; i < bcount; i++) {
                BehaveListener bl = behaveListeners[i];
                try {
                    if (bl.isEnabled()) {
                        if (verbose) System.err.println("Controller.run: invoking behaveListener " + bl.getName());
                        bl.behave();
                    }
                } catch (Exception t) {
                    t.printStackTrace();
                    if (exitOnListenerException)
                        return;
                }
            }

            long etm = System.currentTimeMillis();
            if (etm - ctm > cycleTime && verbose) {
                System.err.println("run: this cycle took " + Long.toString(etm-ctm) + " milliseconds");
            } else {
                try {
                    Thread.sleep(cycleTime - (etm-ctm));
                } catch (InterruptedException e) {
                }
            };            
        }
    };
};

