/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/
package robotcontroller;

/**
 * Represents one shaft encoder.
 *
 * The first 8 ports (sensors 0-7) are augmented with the ability to
 * act as shaft encoder ports that count transitions from low to high.
 *
 * The Controller Encoder class exports an interface to the user that
 * looks much like class AnalogPort. Encoder.updateAllCounts() is the
 * analogue to AnalogPort.updateValues(), and Encoder.getCount() is
 * directly equal to AnalogPort.getValue(). However, Encoders must be
 * enabled through a call to Encoder.enable() before they can be
 * used.
 *
 * Encoders should be configured before use. The Encoder.configure(int
 * low,int high) method sets the thresholds that must be tripped
 * before the Encoder class will register a "count". In particular, in
 * order for a complete low-high cycle to register as a tick and
 * update the Encoder count, the analog sensor value _must_ start
 * below "low" and end above "high".
 * 
 * Hence, users should configure Encoders based on the values their
 * breakbeam sensors are reading -- if the breakbeams are reading
 * 55/240, for example, a reasonable call would be
 * Encoder.configure(80, 200); The low should be above the lowest
 * possible value read by the sensor, and the high should be below the
 * highest possible value read.
 * 
 *
 * If <code>Controller.run</code> (recommended) or
 * <code>Controller.startUserThread</code> (not recommended) is invoked then
 * user code does not have to invoke <code>updateAllCounts</code>, it is
 * automatically invoked once per <code>Controller.cycleTime</code>
 * milliseconds by the Controller thread.  */
public class Encoder implements UpdateListener {
    static Controller rdc;
    int port;
    static int portsInUse;
    static byte counts[] = new byte[Controller.ENC_NUMPORTS];

    /**
     * Creates a new encoder on the specified port.
     * @param port the encoder port
     * @exception ControllerException if the port is in use or out of range.
     *
     * If <code>Controller.run</code> (recommended) or
     * <code>Controller.startUserThread</code> (not recommended) is invoked then
     * user code does not have to invoke <code>updateValues</code>, it is
     * automatically invoked once per <code>Controller.cycleTime</code>
     * milliseconds by the Controller thread.  
     */
    public Encoder(int port) throws ControllerException
    {
        if (rdc == null) {
            rdc = Controller.getController();
            rdc.addUpdateListener(this);
        }
	if (port >= Controller.ENC_NUMPORTS)
	    throw new ControllerException("encoder port out of range");
        int portMask = (1 << port);
        if ((portMask & portsInUse) != 0)
            throw new ControllerException("encoder port already in use");
        if (portsInUse == 0)
            rdc.encoder_enable_all(1);

        this.rdc = rdc;
        this.port = port;
        portsInUse |= portMask;
    };
    /**
     * Frees the encoder port for reuse
     */
    public synchronized void finalize() throws ControllerError
    {
        portsInUse &= ~(1 << port);
        if (portsInUse == 0)
            rdc.encoder_enable_all(0);
    };

    /**
     * Enables all encoders       
     */
    public synchronized static void enableAll() throws ControllerError
    {
        rdc.encoder_enable_all(1);
    };
    /**
     * Disables all encoders
     */
    public synchronized static void disableAll() throws ControllerError
    {
        rdc.encoder_enable_all(0);
    };

    /**
     * Configures a port to count the number of crossings
     * from below <code>low</code> to above <code>high</code>.
     */
    public synchronized void configure(int low, int high) throws ControllerError
    {
        rdc.encoder_configure(port, low, high);
    };

    public synchronized static void resetAllCounts() throws ControllerError
    {
        rdc.encoder_reset_counts();
    };

    public synchronized int getCount() throws ControllerError
    {
        return counts[port];
    };

    /**
     * User code does not have to invoke <code>updateAllCounts</code>, it is
     * automatically invoked once per <code>Controller.cycleTime</code>
     * milliseconds by the Controller thread or by Controller.run.
     * @exception ControllerError if there is a problem accessing the daughtercard.
     */
    public synchronized static void updateAllCounts() throws ControllerError
    {
        if (counts == null) {
            counts = new byte[Controller.ENC_NUMPORTS];
        } else {
            rdc.encoder_get_counts(counts);
        }
    };

    public synchronized void update() throws ControllerError
    {
        if (counts == null) {
            counts = new byte[Controller.ENC_NUMPORTS];
        } else {
            rdc.encoder_get_counts(counts);
        }
    };
};
