/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/

package robotcontroller;

/**
 * Represents one motor port
 */

public class Motor {
    /**
     * ensures that the connection to skfrdc is initialized.
     */
    static Controller rdc = null;

    /**
     * The maximum value to pass to setSpeed
     */
    public final static int MAXFORWARD = 255;

    /**
     * The minimum value to pass to setSpeed for a bidirectional motor
     */
    public final static int MAXREVERSE = -255;

    /**
     * The value to turn off a motor.
     */
    public final static int STOPPED = 0;

    int motor1_num = -1;
    int motor2_num = -1;
    boolean isPair;
    /**
     * is a bitvector with a one for each motor in use
     */
    static int motorsInUse = 0;

    /**
     * Creates a new motor
     * @param motor1_num the number of the motor port to be used
     * @exception ControllerException if motor1_num is out of range or already in use
     * 
     * There is a heat dissipation problem with the motor drivers, so we
     * are restricting them to driving two (either two unidirectional or two bidirectional) motors at a time per chip. 
     * 
     * The allowed values for motor1_num are ??
     */
    public Motor(int motor1_num) throws ControllerException
    {
        if (rdc == null)
            rdc = Controller.getController();
        synchronized (rdc) {
            if ((motorsInUse & (1 << motor1_num)) != 0) {
                throw new ControllerException("motor already in use");
            }
            this.isPair = false;
            this.motor1_num = motor1_num;
            motorsInUse |= (1 << motor1_num);
        }
    };

    /**
     * Creates a new motor pair
     * @param motor1_num the number of the first motor port to be used.  Two consecutive ports will be used.
     * @param isPair create a motor pair if this is true
     * @exception ControllerException if motor1_num is out of range or already in use
     * 
     * There is a heat dissipation problem with the motor drivers, so we
     * are restricting them to driving two (either two unidirectional or two bidirectional) motors at a time per chip. 
     * 
     * The allowed values for motor1_num and motor2_num are ??
     * For a motor pair, <code>motor1_num</code> and <code>motor1_num +
     * 1</code> will be used.  
     */
    public Motor(int motor1_num, int motor2_num) throws ControllerException
    {
        if (rdc == null)
            rdc = Controller.getController();
        synchronized (rdc) {
            int motor_mask = (1 << motor1_num) | (1 << motor2_num);
            if ((motorsInUse & motor_mask) != 0) {
                throw new ControllerException("one or more ports already in use");
            }
            this.motor1_num = motor1_num;
            this.motor2_num = motor2_num;
            this.isPair = true;
            motorsInUse |= motor_mask;
        }
    };

    /**
     * Creates a new motor pair
     * @param motor1_num the number of the first motor port to be used.  Two consecutive ports will be used.
     * @param isPair create a motor pair if this is true
     * @exception ControllerException if motor1_num is out of range or already in use
     * 
     * There is a heat dissipation problem with the motor drivers, so we
     * are restricting them to driving two (either two unidirectional or two bidirectional) motors at a time per chip. 
     * 
     * The allowed values for motor1_num and motor2_num are ??
     * For a motor pair, <code>motor1_num</code> and <code>motor1_num +
     * 1</code> will be used.  
     */
    public Motor(int motor1_num, boolean isPair) throws ControllerException
    {
        if (rdc == null)
            rdc = Controller.getController();
        synchronized (rdc) {
            int motor2_num = (isPair ? motor1_num + 1 : motor1_num);
            int motor_mask = (1 << motor1_num) | (1 << motor2_num);
            if ((motorsInUse & motor_mask) != 0) {
                throw new ControllerException("one or more ports already in use");
            }
            this.motor1_num = motor1_num;
            this.motor2_num = motor2_num;
            this.isPair = isPair;
            motorsInUse |= motor_mask;
        }
    };

    /**
     * Frees the associated motor ports
     */
    public void finalize()
    {
        synchronized (rdc) {
            int motorMask = 0;
            if (isPair)
                motorMask = ((1 << motor1_num) | (1 << motor2_num));
            else
                motorMask = (1 << motor1_num);
            motorsInUse &= ~motorMask;
        }
    };

    /**
     * sets the speed of a motor or motor pair
     *
     * @param speed ranges from 0 to 255 for motors and -255 to 255 for motor pairs
     */
    public void setSpeed(int speed) throws ControllerError
    {
        synchronized (rdc) {
            if (isPair)
                setMotorPair(motor1_num, motor2_num, speed);
            else
                setSpeed(motor1_num, speed);
        }
    };

    /** 
     * sets the speed for a motor
     * @param motor1_num ranges from 0 to 9
     * @param speed ranges from 0 to 255
     */
    private static native void setSpeed(int motor1_num, int speed) throws ControllerError;

    /** 
     * stops all the motors (setting their speeds to 0)
     */
    public static native void stopAll() throws ControllerError;

    /**
     * sets the speed for a <em>motor pair</em>
     *
     *   A 'motor pair' is any two consecutive motor ports. A motor can
     *   be connected to both ends of a motor pair and can then be driven 
     *   both forwards and backwards, by driving just one of the 2 motor ports.
     *   In this command the direction is specified by the sign of the speed.
     *
     *  @param motor1_num is the first motor in the motor pair (valid range 0 to 18).
     *  @param motor2_num is the second motor in the motor pair (valid range 0 to 18).
     *  @param speed  speed to set the motor port to (valid range -255 to 255).
     *                if (speed>=0)  1st port = 'speed' and 2nd port = 0
     *                if (speed<0)   1st port = 0 and 2nd port = -'speed'
     */
    private static native void setMotorPair(int motor1_num, int motor2_num, int speed) throws ControllerError;

    /**
     * Enables the charge pump
     */
    public static void nitroEnable() {
	setSpeed(19, 240);
    };

    /**
     * Disables the charge pump
     */
    public static void nitroDisable() {
	setSpeed(19, 0);
    };
};
